var chai_1 = require('chai');
var vl = require('../src/vl');
var zSchema = require('z-schema');
var inspect = require('util').inspect;
var dl = require('datalib');
var validator = new zSchema();
var vlSchema = require('../src/schema/schema').schema, vgSchema = require('../node_modules/vega/vega-schema.json');
function validateAgainstSchemas(vlspec, done) {
    var isVlValid = validator.validate(vlspec, vlSchema);
    var errors;
    if (!isVlValid) {
        errors = validator.getLastErrors();
        console.log(inspect(errors, { depth: 10, colors: true }));
    }
    chai_1.assert.deepEqual(isVlValid, true);
    var vegaSpec = vl.compile(vlspec);
    var isVgValid = validator.validate(vegaSpec, vgSchema);
    if (!isVgValid) {
        errors = validator.getLastErrors();
        console.log(inspect(errors, { depth: 10, colors: true }));
    }
    chai_1.assert.deepEqual(isVgValid, true);
    if (done) {
        done();
    }
}
describe('Examples', function () {
    var VL_EXAMPLES = dl.json('examples/vlexamples.json');
    var examples = dl.keys(VL_EXAMPLES).reduce(function (specs, groupName) {
        var group = VL_EXAMPLES[groupName];
        return dl.isArray(group) ?
            specs.concat(group) : specs;
    }, []);
    examples.forEach(function (example) {
        it('should be valid and produce valid vega for: ' + example.name, function (done) {
            var jsonData = dl.load({ url: 'examples/' + example.name + '.json' });
            var data = dl.read(jsonData, { type: 'json', parse: 'auto' });
            validateAgainstSchemas(data, done);
        });
    });
});
//# sourceMappingURL=examples.test.js.map